#!/usr/bin/env python
#Author: Mark Feineigle
#Create Date: 2018/03/08
#Last Modify: 2018/03/08
# Mastering Machine Learning with Python in Six Steps page 187
import pandas as pd
import statsmodels.api as sm
from sklearn import preprocessing
import matplotlib.pyplot as plt
import numpy as np

# Load data
df = pd.read_csv('/home/egg/projects/mastering_machine_learning_python/Housing.csv')
# Convert binary fields to numeric boolean fields
lb = preprocessing.LabelBinarizer()

df.driveway = lb.fit_transform(df.driveway)
df.recroom = lb.fit_transform(df.recroom)
df.fullbase = lb.fit_transform(df.fullbase)
df.gashw = lb.fit_transform(df.gashw)
df.airco = lb.fit_transform(df.airco)
df.prefarea = lb.fit_transform(df.prefarea)
# Create dummy variables for stories
df_stories = pd.get_dummies(df['stories'], prefix='stories', drop_first=False)
# Join the dummy variables to the main dataframe
df = pd.concat([df, df_stories], axis=1)
del df['stories']
del df["Unnamed: 0"]
# lets plot correlation matrix using statmodels graphics packages's plot_corr
# create correlation matrix
corr = df.corr()
sm.graphics.plot_corr(corr, xnames=list(corr.columns))
plt.show()

# create a Python list of feature names
#independent_variables = ['lotsize', 'bedrooms', 'bathrms','driveway', 'recroom',
#'fullbase','gashw','airco','garagepl', 'prefarea','stories_1','stories_2','stories_3', 'stories_4']

independent_variables = ['lotsize', 'bathrms','driveway',
'fullbase','gashw','airco','garagepl', 'prefarea','stories_1','stories_3']
del df["stories_4"]
del df["bedrooms"]
del df["recroom"]
del df["stories_2"]

# use the list to select a subset from original DataFrame
X = df[independent_variables]
y = df['price']
thresh = 10
from statsmodels.stats.outliers_influence import variance_inflation_factor
for i in np.arange(0,len(independent_variables)):
  vif = [variance_inflation_factor(X[independent_variables].values, ix) for ix in range(X[independent_variables].shape[1])]
  maxloc = vif.index(max(vif))
  if max(vif) > thresh:
    print "vif :", vif
    print('dropping \'' + X[independent_variables].columns[maxloc] + '\' at index: ' + str(maxloc))
    del independent_variables[maxloc]
  else:
    break
print 'Final variables:', independent_variables

# Split your data set into 80/20 for train/test datasets
from sklearn.cross_validation import train_test_split
X_train, X_test, y_train, y_test = train_test_split(X, y, train_size=.80, random_state=1)

# create a fitted model & print the summary
lm = sm.OLS(y_train, X_train).fit()
print lm.summary()
# make predictions on the testing set
y_train_pred = lm.predict(X_train)
y_test_pred = lm.predict(X_test)
from sklearn import metrics
print "Train MAE: ", metrics.mean_absolute_error(y_train, y_train_pred)
print "Train RMSE: ", np.sqrt(metrics.mean_squared_error(y_train, y_train_pred))
print "Test MAE: ", metrics.mean_absolute_error(y_test, y_test_pred)
print "Test RMSE: ", np.sqrt(metrics.mean_squared_error(y_test, y_test_pred))
